# -*- coding: utf-8 -*-
"""
    flask.ext.social.views
    ~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social views

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""
from importlib import import_module

from flask import Blueprint, current_app, redirect, request, session, \
     after_this_request, abort
from flask.ext.security import current_user, login_required
from flask.ext.security.utils import get_post_login_redirect, login_user, \
     get_url, do_flash
from flask.ext.security.decorators import anonymous_user_required
from werkzeug.local import LocalProxy

from .signals import connection_removed, connection_created, \
     connection_failed, login_completed, login_failed
from .utils import config_value, get_provider_or_404, get_authorize_callback, \
     get_conection_values_from_oauth_response


# Convenient references
_security = LocalProxy(lambda: current_app.extensions['security'])

_social = LocalProxy(lambda: current_app.extensions['social'])

_datastore = LocalProxy(lambda: _social.datastore)

_logger = LocalProxy(lambda: current_app.logger)


def _commit(response=None):
    _datastore.commit()
    return response


@anonymous_user_required
def login(provider_id):
    """Starts the provider login OAuth flow"""
    provider = get_provider_or_404(provider_id)
    callback_url = get_authorize_callback('login', provider_id)
    post_login = request.form.get('next', get_post_login_redirect())
    session['post_oauth_login_url'] = post_login
    return provider.authorize(callback_url)


@login_required
def connect(provider_id):
    """Starts the provider connection OAuth flow"""
    provider = get_provider_or_404(provider_id)
    callback_url = get_authorize_callback('connect', provider_id)
    allow_view = get_url(config_value('CONNECT_ALLOW_VIEW'))
    pc = request.form.get('next', allow_view)
    session[config_value('POST_OAUTH_CONNECT_SESSION_KEY')] = pc
    return provider.authorize(callback_url)


@login_required
def remove_all_connections(provider_id):
    """Remove all connections for the authenticated user to the
    specified provider
    """
    provider = get_provider_or_404(provider_id)

    ctx = dict(provider=provider.name, user=current_user)

    deleted = _datastore.delete_connections(user_id=current_user.get_id(),
                                            provider_id=provider_id)
    if deleted:
        after_this_request(_commit)
        msg = ('All connections to %s removed' % provider.name, 'info')
        connection_removed.send(current_app._get_current_object(),
                                user=current_user._get_current_object(),
                                provider_id=provider_id)
    else:
        msg = ('Unable to remove connection to %(provider)s' % ctx, 'error')

    do_flash(*msg)
    return redirect(request.referrer)


@login_required
def remove_connection(provider_id, provider_user_id):
    """Remove a specific connection for the authenticated user to the
    specified provider
    """
    provider = get_provider_or_404(provider_id)

    ctx = dict(provider=provider.name, user=current_user,
               provider_user_id=provider_user_id)

    deleted = _datastore.delete_connection(user_id=current_user.get_id(),
                                           provider_id=provider_id,
                                           provider_user_id=provider_user_id)

    if deleted:
        after_this_request(_commit)
        msg = ('Connection to %(provider)s removed' % ctx, 'info')
        connection_removed.send(current_app._get_current_object(),
                                user=current_user._get_current_object(),
                                provider_id=provider_id)
    else:
        msg = ('Unabled to remove connection to %(provider)s' % ctx, 'error')

    do_flash(*msg)
    return redirect(request.referrer)


def connect_handler(cv, provider):
    """Shared method to handle the connection process

    :param connection_values: A dictionary containing the connection values
    :param provider_id: The provider ID the connection shoudl be made to
    """
    cv.setdefault('user_id', current_user.get_id())
    connection = _datastore.find_connection(**cv)

    if connection is None:
        after_this_request(_commit)
        connection = _datastore.create_connection(**cv)
        msg = ('Connection established to %s' % provider.name, 'success')
        connection_created.send(current_app._get_current_object(),
                                user=current_user._get_current_object(),
                                connection=connection)
    else:
        msg = ('A connection is already established with %s '
               'to your account' % provider.name, 'notice')
        connection_failed.send(current_app._get_current_object(),
                               user=current_user._get_current_object())

    redirect_url = session.pop(config_value('POST_OAUTH_CONNECT_SESSION_KEY'),
                               get_url(config_value('CONNECT_ALLOW_VIEW')))

    do_flash(*msg)
    return redirect(redirect_url)


def connect_callback(provider_id):
    provider = get_provider_or_404(provider_id)

    def connect(response):
        cv = get_conection_values_from_oauth_response(provider, response)
        if cv is None:
            do_flash('Access was denied by %s' % provider.name, 'error')
            return redirect(get_url(config_value('CONNECT_DENY_VIEW')))
        return cv

    return connect_handler(provider.authorized_handler(connect)(), provider)


@anonymous_user_required
def login_handler(response, provider, query):
    """Shared method to handle the signin process"""

    connection = _datastore.find_connection(**query)

    if connection:
        after_this_request(_commit)
        user = connection.user
        login_user(user)
        key = _social.post_oauth_login_session_key
        redirect_url = session.pop(key, get_post_login_redirect())

        login_completed.send(current_app._get_current_object(),
                             provider=provider, user=user)

        return redirect(redirect_url)

    login_failed.send(current_app._get_current_object(),
                      provider=provider,
                      oauth_response=response)

    next = get_url(_security.login_manager.login_view)
    msg = '%s account not associated with an existing user' % provider.name
    do_flash(msg, 'error')
    return redirect(next)


def login_callback(provider_id):
    try:
        provider = _social.providers[provider_id]
        module = import_module(provider.module)
    except KeyError:
        abort(404)

    def login(response):
        _logger.debug('Received login response from '
                      '%s: %s' % (provider.name, response))

        if response is None:
            do_flash('Access was denied to your %s '
                     'account' % provider.name, 'error')
            return redirect(_security.login_manager.login_view)

        query = dict(provider_user_id=module.get_provider_user_id(response))

        return response, query

    response, query = provider.authorized_handler(login)()
    return login_handler(response, provider, query)


def create_blueprint(state, import_name):
    bp = Blueprint(state.blueprint_name, import_name,
                   url_prefix=state.url_prefix,
                   template_folder='templates')

    bp.route('/login/<provider_id>')(login_callback)

    bp.route('/login/<provider_id>',
             methods=['POST'])(login)

    bp.route('/connect/<provider_id>')(connect_callback)

    bp.route('/connect/<provider_id>',
             methods=['POST'])(connect)

    bp.route('/connect/<provider_id>',
             methods=['DELETE'])(remove_all_connections)

    bp.route('/connect/<provider_id>/<provider_user_id>',
             methods=['DELETE'])(remove_connection)

    return bp



import unittest

from flask.ext.testing import Twill, twill
from tests.test_app.sqlalchemy import create_app as create_sql_app
from tests.test_app.mongoengine import create_app as create_mongo_app


class SocialTest(unittest.TestCase):

    SOCIAL_CONFIG = None
    APP_TYPE = None

    def setUp(self):
        super(SocialTest, self).setUp()
        self.app = self._create_app(self.SOCIAL_CONFIG or None)
        self.app.debug = False
        self.app.config['TESTING'] = True
        self.client = self.app.test_client()

    def _create_app(self, auth_config):
        app_type = self.APP_TYPE or 'sql'
        if app_type == 'sql':
            return create_sql_app(auth_config, False)
        if app_type == 'mongo':
            return create_mongo_app(auth_config, False)

    def _login(self, t):
        t.browser.go(t.url('/login'))
        twill.commands.fv('login_user_form', 'email', 'matt@lp.com')
        twill.commands.fv('login_user_form', 'password', 'password')
        twill.commands.submit(0)

    def _login_provider(self, t, provider):
        t.browser.go(t.url('/login'))
        twill.commands.fv('%s_login_form' % provider, 'login_%s' % provider, '')
        t.browser.submit(1)

    def _start_connect(self, t, provider):
        t.browser.go(t.url('/profile'))
        twill.commands.fv('%s_connect_form' % provider,
                          'connect_' + provider,
                          '')
        t.browser.submit('connect_' + provider)

    def _remove_connection(self, t, provider):
        t.browser.go(t.url('/profile'))
        twill.commands.fv('%s_disconnect_form' % provider,
                          'disconnect_' + provider,
                          '')
        t.browser.submit('disconnect_' + provider)


class TwitterSocialTests(SocialTest):

    def _fill_twitter_oauth(self, t, username, password):
        try:
            twill.commands.fv('oauth_form', 'session[username_or_email]', username)
            twill.commands.fv('oauth_form', 'session[password]', password)
        except:
            pass

    def _authorize_twitter(self, t, username=None, password=None):
        if not 'oauth_form' in t.browser.get_html():
            return
        username = username or self.app.config['TWITTER_USERNAME']
        password = password or self.app.config['TWITTER_PASSWORD']
        self._fill_twitter_oauth(t, username, password)
        form = t.browser.get_form('oauth_form')
        twill.commands.fv('oauth_form', 'cancel', '')
        form.find_control('cancel').disabled = True
        t.browser.submit(0)

    def _deny_twitter(self, t, username, password):
        self._fill_twitter_oauth(username, password)
        t.browser.submit(0)

    def test_connect_twitter(self):
        with Twill(self.app) as t:
            self._login(t)
            self._start_connect(t, 'twitter')
            self._authorize_twitter(t)
            self.assertIn('Connection established to Twitter', t.browser.get_html())

    def test_double_connect_twitter(self):
        with Twill(self.app) as t:
            self._login(t)
            self._start_connect(t, 'twitter')
            self._authorize_twitter(t)
            self._start_connect(t, 'twitter')
            self._authorize_twitter(t)
            self.assertIn('A connection is already established with', t.browser.get_html())

    def test_unconnected_twitter_login(self):
        with Twill(self.app) as t:
            self._login_provider(t, 'twitter')
            self._authorize_twitter(t)
            self.assertIn('Twitter account not associated with an existing user',
                          t.browser.get_html())

    def test_connected_login_with_twitter(self):
        with Twill(self.app) as t:
            self._login(t)
            self._start_connect(t, 'twitter')
            self._authorize_twitter(t)
            t.browser.go(t.url('/logout'))
            self._login_provider(t, 'twitter')
            self._authorize_twitter(t)
            self.assertIn('Profile Page', t.browser.get_html())

    def test_remove_connection(self):
        with Twill(self.app) as t:
            self._login(t)
            self._start_connect(t, 'twitter')
            self._authorize_twitter(t)
            self._remove_connection(t, 'twitter')
            assert 'Connection to Twitter removed' in t.browser.get_html()


class MongoEngineTwitterSocialTests(TwitterSocialTests):
    APP_TYPE = 'mongo'


"""
# Unfortunately Facebook can't be tested because
# the Twill client is not a supported browser.
# Leaving this in for reference
class FacebookSocialTests(SocialTest):

    def _login_facebook(self, t, email=None, password=None):
        email = email or self.app.config['FACEBOOK_EMAIL']
        password = password or self.app.config['FACEBOOK_PASSWORD']
        twill.commands.fv('login_form', 'email', email)
        twill.commands.fv('login_form', 'pass', password)
        t.browser.submit(0)

    def _authorize_facebook(self, t, username=None, password=None):
        if 'login_form' in t.browser.get_html():
            self._login_facebook(t)

        if 'uiserver_form' in t.browser.get_html():
            t.browser.get_form('uiserver_form')
            #t.browser.submit('grant_clicked')
            t.browser.submit(0)

    def test_connect_facebook(self):
        with Twill(self.app) as t:
            self._login(t)
            self._start_connect(t, 'facebook')
            self._authorize_facebook(t)
            assert 'Connection established to Facebook' in t.browser.get_html()
"""



# -*- coding: utf-8 -*-
"""
    flask.ext.social.utils
    ~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social utils

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""
from importlib import import_module

from flask import current_app, url_for, request, abort


def get_provider_or_404(provider_id):
    try:
        return current_app.extensions['social'].providers[provider_id]
    except KeyError:
        abort(404)


def config_value(key, app=None):
    app = app or current_app
    return app.config['SOCIAL_' + key.upper()]


def get_authorize_callback(endpoint, provider_id):
    """Get a qualified URL for the provider to return to upon authorization

    param: endpoint: Absolute path to append to the application's host
    """
    endpoint_prefix = config_value('BLUEPRINT_NAME')
    url = url_for(endpoint_prefix + '.' + endpoint, provider_id=provider_id)
    return request.url_root[:-1] + url


def get_conection_values_from_oauth_response(provider, oauth_response):
    if oauth_response is None:
        return None

    module = import_module(provider.module)

    return module.get_connection_values(oauth_response,
                consumer_key=provider.consumer_key,
                consumer_secret=provider.consumer_secret)


def get_config(app):
    """Conveniently get the social configuration for the specified
    application without the annoying 'SOCIAL_' prefix.

    :param app: The application to inspect
    """
    items = app.config.items()
    prefix = 'SOCIAL_'

    def strip_prefix(tup):
        return (tup[0].replace(prefix, ''), tup[1])

    return dict([strip_prefix(i) for i in items if i[0].startswith(prefix)])


class RecursiveDictionary(dict):
    """RecursiveDictionary provides the methods rec_update and iter_rec_update
    that can be used to update member dictionaries rather than overwriting
    them."""
    def rec_update(self, other, **third):
        """Recursively update the dictionary with the contents of other and
        third like dict.update() does - but don't overwrite sub-dictionaries.

        Example:
        >>> d = RecursiveDictionary({'foo': {'bar': 42}})
        >>> d.rec_update({'foo': {'baz': 36}})
        >>> d
        {'foo': {'baz': 36, 'bar': 42}}
        """
        try:
            iterator = other.iteritems()
        except AttributeError:
            iterator = other
        self.iter_rec_update(iterator)
        self.iter_rec_update(third.iteritems())

    def iter_rec_update(self, iterator):
        for (key, value) in iterator:
            if key in self and \
               isinstance(self[key], dict) and isinstance(value, dict):
                self[key] = RecursiveDictionary(self[key])
                self[key].rec_update(value)
            else:
                self[key] = value



# -*- coding: utf-8 -*-
"""
    flask.ext.social.providers.foursquare
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social foursquare code

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import foursquare

config = {
    'id': 'foursquare',
    'name': 'foursquare',
    'install': 'pip install foursquare',
    'module': 'flask_social.providers.foursquare',
    'base_url': 'https://api.foursquare.com/v2/',
    'request_token_url': None,
    'access_token_url': 'https://foursquare.com/oauth2/access_token',
    'authorize_url': 'https://foursquare.com/oauth2/authenticate',
    'access_token_params': {
        'grant_type': 'authorization_code'
    },
    'request_token_params': {
        'response_type': 'code'
    }
}


def get_api(connection, **kwargs):
    return foursquare.Foursquare(
            access_token=getattr(connection, 'access_token'))


def get_provider_user_id(response, **kwargs):
    if response:
        api = foursquare.Foursquare(
            access_token=getattr(response, 'access_token'))
        return api.users()['user']['id']
    return None


def get_connection_values(response, **kwargs):
    if not response:
        return None

    access_token = response['access_token']
    api = foursquare.Foursquare(access_token=access_token)
    user = api.users()['user']
    profile_url = 'http://www.foursquare.com/user/' + user['id']
    image_url = user['photo']

    return dict(
        provider_id=config['id'],
        provider_user_id=user['id'],
        access_token=access_token,
        secret=None,
        display_name=profile_url.split('/')[-1:][0],
        profile_url=profile_url,
        image_url=image_url
    )



# -*- coding: utf-8 -*-
"""
    flask.ext.social.datastore
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains an abstracted social connection datastore.

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""

from flask_security.datastore import SQLAlchemyDatastore, MongoEngineDatastore


class ConnectionDatastore(object):
    """Abstracted oauth connection datastore. Always extend this class and
    implement parent methods

    :param db: An instance of a configured databse manager from a Flask
               extension such as Flask-SQLAlchemy or Flask-MongoEngine"""

    def __init__(self, connection_model):
        self.connection_model = connection_model

    def find_connection(self, **kwargs):
        raise NotImplementedError

    def find_connections(self, **kwargs):
        raise NotImplementedError

    def create_connection(self, **kwargs):
        return self.put(self.connection_model(**kwargs))

    def delete_connection(self, **kwargs):
        """Remove a single connection to a provider for the specified user
        """
        conn = self.find_connection(**kwargs)
        if not conn:
            return False
        self.delete(conn)
        return True

    def delete_connections(self, **kwargs):
        """Remove a single connection to a provider for the specified user
        """
        rv = False
        for c in self.find_connections(**kwargs):
            self.delete(c)
            rv = True
        return rv


class SQLAlchemyConnectionDatastore(SQLAlchemyDatastore, ConnectionDatastore):
    """A SQLAlchemy datastore implementation for Flask-Social."""

    def __init__(self, db, connection_model):
        SQLAlchemyDatastore.__init__(self, db)
        ConnectionDatastore.__init__(self, connection_model)

    def _query(self, **kwargs):
        return self.connection_model.query.filter_by(**kwargs)

    def find_connection(self, **kwargs):
        return self._query(**kwargs).first()

    def find_connections(self, **kwargs):
        return self._query(**kwargs)


class MongoEngineConnectionDatastore(MongoEngineDatastore, ConnectionDatastore):
    """A MongoEngine datastore implementation for Flask-Social."""

    def __init__(self, db, connection_model):
        MongoEngineDatastore.__init__(self, db)
        ConnectionDatastore.__init__(self, connection_model)

    def _query(self, **kwargs):
        return self.connection_model.objects(**kwargs)

    def find_connection(self, **kwargs):
        return self._query(**kwargs).first()

    def find_connections(self, **kwargs):
        return self._query(**kwargs)



# -*- coding: utf-8 -*-
"""
    flask.ext.social.providers.google
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social google code

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import httplib2
import oauth2client.client as googleoauth
import apiclient.discovery as googleapi

config = {
    'id': 'google',
    'name': 'Google',
    'install': 'pip install google-api-python-client',
    'module': 'flask_social.providers.google',
    'base_url': 'https://www.google.com/accounts/',
    'authorize_url': 'https://accounts.google.com/o/oauth2/auth',
    'access_token_url': 'https://accounts.google.com/o/oauth2/token',
    'request_token_url': None,
    'access_token_method': 'POST',
    'access_token_params': {
        'grant_type': 'authorization_code'
    },
    'request_token_params': {
        'response_type': 'code',
        'scope': 'https://www.googleapis.com/auth/plus.me'
    }
}


def get_api(connection, **kwargs):
    credentials = googleoauth.AccessTokenCredentials(
        access_token=getattr(connection, 'access_token'),
        user_agent=''
    )

    http = httplib2.Http()
    http = credentials.authorize(http)
    return googleapi.build('plus', 'v1', http=http)


def get_provider_user_id(response, **kwargs):
    if response:
        credentials = googleoauth.AccessTokenCredentials(
            access_token=response['access_token'],
            user_agent=''
        )

        http = httplib2.Http()
        http = credentials.authorize(http)
        api = googleapi.build('plus', 'v1', http=http)
        profile = api.people().get(userId='me').execute()
        return profile['id']
    return None


def get_connection_values(response, **kwargs):
    if not response:
        return None

    access_token = response['access_token']

    credentials = googleoauth.AccessTokenCredentials(
        access_token=access_token,
        user_agent=''
    )

    http = httplib2.Http()
    http = credentials.authorize(http)
    api = googleapi.build('plus', 'v1', http=http)
    profile = api.people().get(userId='me').execute()

    return dict(
        provider_id=config['id'],
        provider_user_id=profile['id'],
        access_token=access_token,
        secret=None,
        display_name=profile['displayName'],
        profile_url=profile['url'],
        image_url=profile['image']['url']
    )



# -*- coding: utf-8 -*-
"""
    flask.ext.social.providers.facebook
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social facebook code

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import facebook

config = {
    'id': 'facebook',
    'name': 'Facebook',
    'install': 'pip install facebook-sdk',
    'module': 'flask_social.providers.facebook',
    'base_url': 'https://graph.facebook.com/',
    'request_token_url': None,
    'access_token_url': '/oauth/access_token',
    'authorize_url': 'https://www.facebook.com/dialog/oauth',
    'request_token_params': {
        'scope': 'email'
    }
}


def get_api(connection, **kwargs):
    return facebook.GraphAPI(getattr(connection, 'access_token'))


def get_provider_user_id(response, **kwargs):
    if response:
        graph = facebook.GraphAPI(response['access_token'])
        profile = graph.get_object("me")
        return profile['id']
    return None


def get_connection_values(response, **kwargs):
    if not response:
        return None

    access_token = response['access_token']
    graph = facebook.GraphAPI(access_token)
    profile = graph.get_object("me")
    profile_url = "http://facebook.com/profile.php?id=%s" % profile['id']
    image_url = "http://graph.facebook.com/%s/picture" % profile['id']

    return dict(
        provider_id=config['id'],
        provider_user_id=profile['id'],
        access_token=access_token,
        secret=None,
        display_name=profile['username'],
        profile_url=profile_url,
        image_url=image_url
    )



# -*- coding: utf-8 -*-
"""
    flask.ext.social.core
    ~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social core

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""
from importlib import import_module

from flask import current_app
from flask.ext.oauth import OAuthRemoteApp as BaseRemoteApp
from flask.ext.security import current_user
from werkzeug.local import LocalProxy

from .utils import get_config, RecursiveDictionary
from .views import create_blueprint

_security = LocalProxy(lambda: current_app.extensions['security'])

_social = LocalProxy(lambda: current_app.extensions['social'])

_datastore = LocalProxy(lambda: _social.datastore)

_logger = LocalProxy(lambda: current_app.logger)

default_config = {
    'SOCIAL_BLUEPRINT_NAME': 'social',
    'SOCIAL_URL_PREFIX': None,
    'SOCIAL_CONNECT_ALLOW_VIEW': '/',
    'SOCIAL_CONNECT_DENY_VIEW': '/',
    'SOCIAL_POST_OAUTH_CONNECT_SESSION_KEY': 'post_oauth_connect_url',
    'SOCIAL_POST_OAUTH_LOGIN_SESSION_KEY': 'post_oauth_login_url',
    'SOCIAL_APP_URL': 'http://localhost'
}


class OAuthRemoteApp(BaseRemoteApp):

    def __init__(self, id, module, install, *args, **kwargs):
        BaseRemoteApp.__init__(self, None, **kwargs)
        self.id = id
        self.module = module

    def get_connection(self):
        return _social.datastore.find_connection(provider_id=self.id,
                                                 user_id=current_user.id)

    def get_api(self):
        module = import_module(self.module)
        connection = self.get_connection()
        if connection is None:
            return None
        return module.get_api(connection=connection,
                              consumer_key=self.consumer_key,
                              consumer_secret=self.consumer_secret)


def _get_state(app, datastore, providers, **kwargs):
    config = get_config(app)

    for key in providers.keys():
        config.pop(key.upper())

    for key, value in config.items():
        kwargs[key.lower()] = value

    kwargs.update(dict(
        app=app,
        datastore=datastore,
        providers=providers))

    return _SocialState(**kwargs)


class _SocialState(object):

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key.lower(), value)

    def __getattr__(self, name):
        try:
            return self.providers[name]
        except KeyError:
            msg = "'_SocialState' object has no attribute '%s'" % name
            raise AttributeError(msg)


def _get_token():
    # Social doesn't use the builtin remote method calls feature of the
    # Flask-OAuth extension so we don't need to return a token. This does,
    # however, need to be configured
    return None


class Social(object):

    def __init__(self, app=None, datastore=None):
        self.app = app
        self.datastore = datastore

        if app is not None and datastore is not None:
            self._state = self.init_app(app, datastore)

    def init_app(self, app, datastore=None):
        """Initialize the application with the Social extension

        :param app: The Flask application
        :param datastore: Connection datastore instance
        """

        datastore = datastore or self.datastore

        for key, value in default_config.items():
            app.config.setdefault(key, value)

        providers = dict()

        for key, config in app.config.items():
            if not key.startswith('SOCIAL_') or key in default_config:
                continue

            config = RecursiveDictionary(config)
            suffix = key.lower().replace('social_', '')
            default_module_name = 'flask_social.providers.%s' % suffix
            module_name = config.get('module', default_module_name)

            module = import_module(module_name)
            config.rec_update(module.config)

            providers[config['id']] = OAuthRemoteApp(**config)
            providers[config['id']].tokengetter(_get_token)

        state = _get_state(app, datastore, providers)

        app.register_blueprint(create_blueprint(state, __name__))
        app.extensions['social'] = state

        return state

    def __getattr__(self, name):
        return getattr(self._state, name, None)



# -*- coding: utf-8 -*-
"""
    flask.ext.social.providers.twitter
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social twitter code

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import twitter

config = {
    'id': 'twitter',
    'name': 'Twitter',
    'install': 'pip install python-twitter',
    'module': 'flask_social.providers.twitter',
    'base_url': 'http://api.twitter.com/1/',
    'request_token_url': 'https://api.twitter.com/oauth/request_token',
    'access_token_url': 'https://api.twitter.com/oauth/access_token',
    'authorize_url': 'https://api.twitter.com/oauth/authenticate'
}


def get_api(connection, **kwargs):
    return twitter.Api(consumer_key=kwargs.get('consumer_key'),
                       consumer_secret=kwargs.get('consumer_secret'),
                       access_token_key=connection.access_token,
                       access_token_secret=connection.secret)


def get_provider_user_id(response, **kwargs):
    return response['user_id'] if response else None


def get_connection_values(response=None, **kwargs):
    if not response:
        return None

    api = twitter.Api(consumer_key=kwargs.get('consumer_key'),
                      consumer_secret=kwargs.get('consumer_secret'),
                      access_token_key=response['oauth_token'],
                      access_token_secret=response['oauth_token_secret'])

    user = api.VerifyCredentials()

    return dict(
        provider_id=config['id'],
        provider_user_id=str(user.id),
        access_token=response['oauth_token'],
        secret=response['oauth_token_secret'],
        display_name='@%s' % user.screen_name,
        profile_url="http://twitter.com/%s" % user.screen_name,
        image_url=user.profile_image_url
    )



"""
Flask-Social
============

Oauth provider login and APIs for use with
`Flask-Security <http://packages.python.org/Flask-Security/>`_

Resources
---------

- `Documentation <http://packages.python.org/Flask-Social/>`_
- `Issue Tracker <http://github.com/mattupstate/flask-social/issues>`_
- `Code <http://github.com/mattupstate/flask-social/>`_
- `Development Version
  <http://github.com/mattupstate/flask-rq/zipball/develop#egg=Flask-Social-dev>`_

"""
from setuptools import setup

setup(
    name='Flask-Social',
    version='1.5.3',
    url='https://github.com/mattupstate/flask-social',
    license='MIT',
    author='Matthew Wright',
    author_email='matt@nobien.net',
    description='Simple OAuth provider integration for Flask-Security',
    long_description=__doc__,
    packages=[
        'flask_social',
        'flask_social.providers'
    ],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask-Security==1.5.0',
        'Flask-OAuth==0.12'
    ],
    test_suite='nose.collector',
    tests_require=[
        'nose',
        'Flask-Testing',
        'Flask-SQLAlchemy',
        'Flask-MongoEngine'
    ],
    dependency_links=[
        'http://github.com/mattupstate/flask-security/tarball/develop#egg=Flask-Security-1.3.0-dev'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



from unittest import TestCase
from flask_social.core import _SocialState


class FlaskSocialUnitTests(TestCase):

    def test_social_state_raises_attribute_error(self):
        state = _SocialState(providers={})
        self.assertRaises(AttributeError, lambda: state.something)



# -*- coding: utf-8 -*-
"""
    flask.ext.social.signals
    ~~~~~~~~~~~~~~~~~~~~~~~~

    This module contains the Flask-Social signals

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""

import blinker

signals = blinker.Namespace()

connection_created = signals.signal("connection-created")

connection_failed = signals.signal("connection-failed")

connection_removed = signals.signal("connection-removed")

login_failed = signals.signal("login-failed")

login_completed = signals.signal("login-success")



# -*- coding: utf-8 -*-
"""
    flask.ext.social
    ~~~~~~~~~~~~~~~~

    Flask-Social is a Flask extension that aims to add simple OAuth provider
    integration for Flask-Security

    :copyright: (c) 2012 by Matt Wright.
    :license: MIT, see LICENSE for more details.
"""

__version__ = '1.5.3'

from .core import Social
from .datastore import SQLAlchemyConnectionDatastore, \
     MongoEngineConnectionDatastore
from .signals import connection_created, connection_failed, login_failed, \
     connection_removed, login_completed
